if (select(2, UnitClass("player"))) ~= "ROGUE" then return end

--[ Addon Declaration ] --
Envenomous = LibStub("AceAddon-3.0"):NewAddon("Envenomous", "AceEvent-3.0", "AceConsole-3.0", "LibBars-1.0")
local addon = Envenomous
local media = LibStub("LibSharedMedia-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

--[ Localize Stuff ]--
local _G = getfenv(0)
local UnitGUID = _G.UnitGUID
local UnitExists = _G.UnitExists
local GetSpellInfo = _G.GetSpellInfo
local GetTime = _G.GetTime
local pairs = _G.pairs
local tremove = table.remove
local _,_,icon = GetSpellInfo(57970)
local pName = UnitName("player")

local poisons = {
	[26968] = true,
	[27187] = true,
	[57969] = true,
	[57970] = true,
}

local tracking = {}

local function GetLSMIndex(t, value)
	for k, v in pairs(media:List(t)) do
	if v == value then
		return k
		end
	end
	return nil
end

--[ Options/Profile Methods ]--
local defaults = {
	profile = {
		width = 250,
		height = 16,
		scale = 1,
		texture = "Banto Bar",
		font = "Emblem",
		position = {},
	},
}

local options = {
	type = "group",
	args = {
                toggle = {
			type = "execute",
			name = "Toggle anchor",
			desc = "Toggle the bar anchor frame",
			func = function()
				       addon:ToggleAnchor()
			       end,
		},
                runtest = {
			type = "execute",
			name = "Test",
			desc = "Start a test bar",
			func = function()
				       addon:TestBar()
			       end,
		},

		bars = {
			type = "group",
			name = "Bar settings",
			desc = "Bar settings",
			args = {
		                width = {
		                        type = "range",
		                        name = "Width",
		                        desc = "Set the width of the bars",
		                        get = function() return addon.db.profile.width end,
		                        set = function(info, v)
						      addon.db.profile.width = v
						      addon:UpdateAnchor()
					      end,
		                        min = 50,
		                        max = 500,
		                        step = 5,
		                        isPercent = false,
					order = 2,
		                },
		                height = {
		                        type = "range",
		                        name = "Height",
		                        desc = "Set the height of the bars",
		                        get = function() return addon.db.profile.height end,
		                        set = function(info, v)
						      addon.db.profile.height = v
						      addon:UpdateAnchor()
					      end,
		                        min = 10,
		                        max = 100,
		                        step = 1,
		                        isPercent = false,
					order = 3,
		                },
		                scale = {
		                        type = "range",
		                        name = "Scale",
		                        desc = "Set the scale of the bars",
		                        get = function() return addon.db.profile.scale end,
		                        set = function(info, v)
						      addon.db.profile.scale = v
						      addon:UpdateAnchor()
					      end,
		                        min = 0.1,
		                        max = 5,
		                        step = 0.01,
		                        isPercent = true,
					order = 4,
		                },
				texture = {
					type = "select",
					name = "Texture",
					desc = "Set the texture for the timer bars",
					values = media:List("statusbar"),
					get = function(info) return GetLSMIndex("statusbar", addon.db.profile.texture) end,
					set = function(info, value) 
							addon.db.profile.texture = media:List("statusbar")[value]
							addon:UpdateAnchor()
						end,
					order = 6,
				},
				font = {
					type = "select",
		                        name = "Font",
		                        desc = "Set the bar font",
					values = media:List("font"),
					get = function(info) return GetLSMIndex("font", addon.db.profile.font) end,
					set = function(info, value)
							addon.db.profile.font = media:List("font")[value]
							addon:UpdateAnchor()
						end,
					order = 7,
				},
				fontsize = {
					type = "range",
		                        name = "Font Size",
		                        desc = "Set the bar font size",
					min = 7,
					max = 48,
					step = 1,
					get = function(info) return addon.db.profile.fontsize end,
					set = function(info, value)
							addon.db.profile.fontsize = value
							addon:UpdateAnchor()
						end,
					order = 8,
				},
			},
			order = 10,
		},
	},
}

AceConfig:RegisterOptionsTable("Envenomous", options)

--[ Addon Setup Methods ]--
function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("EnvenomousDB", defaults, "Default")

	self:RegisterChatCommand("Envenomous", function() LibStub("AceConfigDialog-3.0"):Open("Envenomous") end )
	local optFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Envenomous", "Envenomous")
	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)

	self.db.RegisterCallback(self, "OnProfileChanged", "UpdateAnchor")
	self.db.RegisterCallback(self, "OnProfileCopied", "UpdateAnchor")
	self.db.RegisterCallback(self, "OnProfileReset", "UpdateAnchor")	

        media.RegisterCallback(self, "LibSharedMedia_Registered", "UpdateAnchor")
        media.RegisterCallback(self, "LibSharedMedia_SetGlobal", "UpdateAnchor")

	self.anchor = self:NewBarGroup("Envenomous", nil, self.db.profile.width, self.db.profile.height)
	self.anchor:ClearAllPoints()
	self.anchor.RegisterCallback(self, "AnchorClicked")
	self.anchor:HideAnchor()
	self:UpdateAnchor()

end

function addon:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
end

function addon:OnDisable()
	self:UnregisterAllEvents()
end

--[ Timer Bar Setup Methods ]--
function addon:AnchorClicked(callback, group, button)
	self:SavePosition(group.id)
	if button == "RightButton" then
		group:HideAnchor()
		group:Lock()
	end
end

function addon:ToggleAnchor()
	if self.anchor.button:IsVisible() then
		self.anchor:HideAnchor()
		self.anchor:Lock()
	else
		self.anchor:ShowAnchor()
		self.anchor:Unlock()
	end
end

function addon:SavePosition(anchor)
	local p,_,pr,px,py = self.anchor:GetPoint()
	self.db.profile.position.p = p
	self.db.profile.position.pr = pr
	self.db.profile.position.px = px
	self.db.profile.position.py = py
end

function addon:UpdateAnchor()
	self.anchor:SetTexture(media:Fetch("statusbar", self.db.profile.texture))
	self.anchor:SetFont(media:Fetch("font", self.db.profile.font), self.db.profile.fontsize)
	self.anchor:SetWidth(self.db.profile.width)
	self.anchor:SetHeight(self.db.profile.height)
	self.anchor:SetFlashPeriod(5)
	self.anchor:ClearAllPoints()
	self.anchor:SetPoint(self.db.profile.position.p or "CENTER", UIParent, self.db.profile.position.pr or "CENTER", self.db.profile.position.px or 0, self.db.profile.position.py or 0)
	self.anchor:SetScale(self.db.profile.scale)
	self.anchor:SetColorAt(1.00, 0.0, 1.0, 0.0, 1)
end

--[ Bar Management Methods ]--
function addon:TestBar()
	local start = GetTime()
	self.anchor:NewTimerBar("Envenomous", "Test Bar", 10, 10, icon)
end

function addon:StartBar(length)
	self.anchor:NewTimerBar("Envenomous", "Deadly Poison", length, 3, icon)
end

function addon:ResetBar()
	self:CancelBar()
	self:StartBar(3)
end

function addon:CancelBar()
	local bars = self.anchor:GetBars()
	if type(bars) == "table" then
		for k, v in pairs(bars) do
			self.anchor:RemoveBar(k)
		end
	end
end

--[ Event Handlers ]--
function addon:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellID, spellName, spellFlags, ...)
	if (srcName ~= pName) or (poisons[spellID] ~= true) then return end

	if eventType == "SPELL_AURA_APPLIED" then
		tracking[dstGUID] = GetTime()
		if dstGUID == UnitGUID("target") then 
			self:StartBar(3)
		end
	elseif eventType == "SPELL_PERIODIC_DAMAGE" then 
		tracking[dstGUID] = GetTime()
		if (dstGUID == UnitGUID("target")) then
			self:ResetBar()
		end
	elseif eventType == "SPELL_AURA_REMOVED" then
		tremove(tracking, dstGUID)
		if dstGUID == UnitGUID("target") then
			self:CancelBar()
		end
	end
end

function addon:PLAYER_TARGET_CHANGED()
	self:CancelBar()
	if UnitExists("target") then
		guid = UnitGUID("target")
		if tracking[guid] then
			length = ((GetTime() - tracking[guid]) % 3.0)
			self:StartBar(length)
		end
	end
end
